/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muse.example;

import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipParticipantType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipTypeType;
import org.xmlsoap.schemas.ws.x2004.x08.addressing.EndpointReferenceType;
import javax.xml.namespace.QName;

/**
 * @author Kinga Dziembowski
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ExampleUtils
{
   /**
    * DOCUMENT_ME
    *
    * @param relationshipRemove DOCUMENT_ME
    * @param relationshipFromProp DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static boolean CompareRelationships( RelationshipType relationshipRemove,
                                               RelationshipType relationshipFromProp )
   {
      // get type of relationship to be removed
      RelationshipTypeType typeRemove = relationshipRemove.getType(  );
      RelationshipTypeType type = relationshipFromProp.getType(  );

      if ( !compareType( typeRemove, type ) )
      {
         return false;
      }

      if ( !compareParticipants( relationshipRemove, relationshipFromProp ) )
      {
         return false;
      }

      return true;
   }

   /**
    * DOCUMENT_ME
    *
    * @param firstParticipant DOCUMENT_ME
    * @param secondParticipant DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static boolean compare( RelationshipParticipantType firstParticipant,
                                  RelationshipParticipantType secondParticipant )
   {
      EndpointReferenceType firstEpr  = firstParticipant.getManageabilityEndpointReferenceArray( 0 );
      EndpointReferenceType secondEpr = secondParticipant.getManageabilityEndpointReferenceArray( 0 );
      if ( firstEpr.getAddress(  ).getStringValue(  ).compareTo( secondEpr.getAddress(  ).getStringValue(  ) ) != 0 )
      {
         return false;
      }

      XmlObject[] firstchildElems  = XmlBeanUtils.getChildElements( firstEpr.getReferenceProperties(  ) );
      XmlObject[] secondchildElems = XmlBeanUtils.getChildElements( secondEpr.getReferenceProperties(  ) );
      if ( firstchildElems.length != secondchildElems.length )
      {
         return false;
      }

      if ( firstchildElems.length > 0 )
      {
         for ( int i = 0; i < firstchildElems.length; i++ )
         {
            XmlObject obj1   = firstchildElems[i];
            QName     tag1   = XmlBeanUtils.getName( obj1 );
            String    value1 = XmlBeanUtils.getValue( obj1 );

            XmlObject obj2   = secondchildElems[i];
            QName     tag2   = XmlBeanUtils.getName( obj2 );
            String    value2 = XmlBeanUtils.getValue( obj2 );
            if ( tag1.toString(  ).compareTo( tag2.toString(  ) ) != 0 )
            {
               return false;
            }

            if ( value1.compareTo( value2 ) != 0 )
            {
               return false;
            }
         }
      }

      if ( firstParticipant.getRole(  ).compareTo( secondParticipant.getRole(  ) ) != 0 )
      {
         return false;
      }

      if ( firstParticipant.getResourceId(  ).compareTo( secondParticipant.getResourceId(  ) ) != 0 )
      {
         return false;
      }

      return true;
   }

   /**
    * DOCUMENT_ME
    *
    * @param relationshipRemove DOCUMENT_ME
    * @param relationshipFromProp DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static boolean compareParticipants( RelationshipType relationshipRemove,
                                              RelationshipType relationshipFromProp )
   {
      //get participants arrays
      RelationshipParticipantType[] participantsRemove = relationshipRemove.getParticipantArray(  );
      RelationshipParticipantType[] participants = relationshipFromProp.getParticipantArray(  );

      //The correct relationship needs to have 2 Participants
      // The relationship match require this same number of participants
      if ( ( ( participantsRemove.length < 2 ) || ( participants.length < 2 ) )
           || ( participantsRemove.length != participants.length ) )
      {
         return false;
      }

      for ( int i = 0; i < participants.length; i++ )
      {
         if ( !compare( participantsRemove[i], participants[i] ) )
         {
            return false;
         }
      }

      return true;
   }

   /**
    * Compares the Relationship Type
    * @param RelationshipTypeType typeRemove
    * @param RelationshipTypeType typeFromProp
    * @return true if the Types are this same, false if the Types are different
    */
   public static boolean compareType( RelationshipTypeType typeRemove,
                                      RelationshipTypeType typeFromProp )
   {
      XmlObject[] childRemove = XmlBeanUtils.getChildElements( typeRemove );
      XmlObject[] childElems = XmlBeanUtils.getChildElements( typeFromProp );

      if ( ( childElems.length > 0 ) && ( childRemove.length > 0 ) )
      {
         XmlObject obj       = childElems[0];
         XmlObject objRemove = childRemove[0];
         QName     tag       = XmlBeanUtils.getName( obj );
         QName     tagRemove = XmlBeanUtils.getName( objRemove );

         // if the relationship Types are not qual - the relationships are not equal 
         if ( tagRemove.toString(  ).compareTo( tag.toString(  ) ) != 0 )
         {
            return false;
         }
      }

      return true;
   }
}