/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muse.example.businessprocesstype.backend;

import java.util.ArrayList;
import java.util.Random;

/**
 * @author Kinga Dziembowski
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class BusinessProcessInstance
{
   private String    m_Id;
   private Random    m_random             = new Random(  );
   private Random    m_time               = new Random(  );
   private long      m_cumulativeTime     = 0L;
   private long      m_totProcesses       = 0L;
   private int       m_runningProcesses   = 0;
   private ArrayList m_allprocessesIds    = new ArrayList(  );
   private ArrayList m_runProcesses       = new ArrayList(  );
   private ArrayList m_suspProcesses      = new ArrayList(  );
   private ArrayList m_termProcesses      = new ArrayList(  );
   private ArrayList m_abortedProcesses   = new ArrayList(  );
   private ArrayList m_completedProcesses = new ArrayList(  );

   /**
    * Creates a new {@link BusinessProcessInstance} object.
    *
    * @param id DOCUMENT_ME
    */
   public BusinessProcessInstance( String id )
   {
      m_Id = id;
      int size = 10 + m_random.nextInt( 20 );
      for ( int i = 0; i <= size; i++ )
      {
         String processId = Integer.toString( i );
         m_allprocessesIds.add( processId );
         m_runProcesses.add( processId );
      }

      m_totProcesses = m_runningProcesses = m_allprocessesIds.size(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public long getCumulativeExecutionTime(  )
   {
      m_cumulativeTime += m_time.nextInt( 30 );
      return m_cumulativeTime;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public int getNumberOfRunningProcesses(  )
   {
      return m_runProcesses.size(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public int getNumberOfSuspendedProcesses(  )
   {
      return m_suspProcesses.size(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public long getTotalNumberOfAbortedProcesses(  )
   {
      return m_abortedProcesses.size(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public long getTotalNumberOfCompletedProcesses(  )
   {
      return m_completedProcesses.size(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public long getTotalNumberOfTerminatedProcesses(  )
   {
      return m_termProcesses.size(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param processId DOCUMENT_ME
    */
   public void resume( String processId )
   {
      //		 Check if the id is a valid Id
      boolean exists = m_allprocessesIds.contains( processId );
      if ( exists )
      {
         // Check if the process is not already suspended
         if ( m_suspProcesses.contains( processId ) )
         {
            //Remove the process to suspended list
            m_suspProcesses.remove( processId );

            // Move the process from running list
            m_runProcesses.add( processId );
         }
      }
   }
   
   /**
    * DOCUMENT_ME
    *
    * @param processId DOCUMENT_ME
    */
   public void complete( String processId )
   {
      //		 Check if the id is a valid Id
      boolean exists = m_allprocessesIds.contains( processId );
      if ( exists )
      {
         // Check if the process is running
         if ( m_runProcesses.contains( processId ) )
         {
            //Remove the process to running list
         	m_runProcesses.remove( processId );

            // Move the process to completed list
            m_completedProcesses.add( processId );
         }
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param processId DOCUMENT_ME
    */
   public void suspend( String processId )
   {
      // Check if the id is a valid Id
      boolean exists = m_allprocessesIds.contains( processId );
      if ( exists )
      {
         // Check if the process is not already suspended
         if ( !m_suspProcesses.contains( processId ) )
         {
         	 // remove the process from running list
            m_runProcesses.remove( processId );
            
         	//Move the process to suspended list
            m_suspProcesses.add( processId );
           
         }
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param processId DOCUMENT_ME
    */
   public void terminate( String processId )
   {
      // Check if the id is a valid Id
      boolean exists = m_allprocessesIds.contains( processId );
      if ( exists )
      {
         // Check if the process is not already suspended
         if ( m_suspProcesses.contains( processId ) )
         {
            //Remove the process to suspended list
            m_suspProcesses.remove( processId );
         }

         if ( m_runProcesses.contains( processId ) )
         {
            // Move the process from running list
            m_runProcesses.remove( processId );
         }
         m_termProcesses.add( processId );
      }
   }
}