package org.apache.ws.muse.example.integrationserver;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.mows.v1_0.MowsConstants;
import org.apache.ws.mows.v1_0.capability.MetricsCapability;
import org.apache.ws.mows.v1_0.capability.OperationalStatusCapability;
import org.apache.ws.muse.example.ExampleConstants;
import org.apache.ws.muse.example.ExampleUtils;
import org.apache.ws.muse.example.resourceadmin.properties.CreateParamsType;
import org.apache.ws.muws.MuwsUtils;
import org.apache.ws.muws.v1_0.capability.DescriptionCapability;
import org.apache.ws.muws.v1_0.capability.IdentityCapability;
import org.apache.ws.muws.v1_0.capability.ManageabilityCharacteristicsCapability;
import org.apache.ws.muws.v1_0.capability.RelationshipsCapability;
import org.apache.ws.muws.v1_0.topics.ManagementEventTopic;
import org.apache.ws.muws.v1_0.topics.impl.XmlBeansManagementEventTopicImpl;
import org.apache.ws.notification.topics.TopicSpace;
import org.apache.ws.notification.topics.TopicSpaceSet;
import org.apache.ws.notification.topics.impl.TopicSpaceImpl;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CurrentTimeDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipParticipantType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipType;

import java.util.Calendar;

/**
 * A Integrationserver WS-Resource.
 * <p/>
 * NOTE: This class is generated but IS meant to be modified.
 */
public class IntegrationserverResource extends AbstractIntegrationserverResource

{

     /**
     * A NamespaceVerionHolder which maintains the QNames of Spec Wsdls
     */
    public static final org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl SPEC_NAMESPACE_SET = new org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl();


     /**  The value for the ImplementationProfile property **/

    public static final String RESOURCE_TYPE = "IntegrationServer";

    /**  The value for the Caption property **/

    public static final String CAPTION = "Integration Server";

    /**  The value for the Version property **/

    public static final String VERSION = "1.0";

    /** The value for the Description property */

    public static final String DESCRIPTION = "Integration Server root for the WSDM Application management";

    public static final String RESOURCE_ID = "Integration_Server";

    private static final Log LOG = LogFactory.getLog( IntegrationserverResource.class.getName() );

    public IntegrationserverResource( Object resourceId, CreateParamsType params)
    {
    	setID( resourceId );
    	// params can have any important information for the creation of Resource
    	// in this example it is not used
    }

    /**
     * Initializes this resource's state (properties, etc.).
     */
    public void init()
    {
        super.init();

        /**
		 * The ResourcePropertySet which contains all the defined ResourceProperties
		 */
		org.apache.ws.resource.properties.ResourcePropertySet resourcePropertySet = getResourcePropertySet();
		org.apache.ws.resource.properties.ResourceProperty resourceProperty = null;


	try{

		// Resource Property {http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd}FixedTopicSet is implemented by the framework.
		// Resource Property {http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd}Topic is implemented by the framework.
		// Resource Property {http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd}TopicExpressionDialects is implemented by the framework.


	     /**
	     * Initializes the Topic, TopicExpressionDialects and FixedTopicSet resource properties
		 * </br>
		 * FixedTopicSet will be set based on the value returned from the {@see TopicSpaceSet#isFixed()} method.
		 * </br>
		 * Topic will have all root topics in the TopicSpaceSet, set to Simple dialect AND all child topics set to
		 * Concrete dialect.
		 * </br>
		 *
     	 * TopicExpressionDialects will be set to the engine's known Topic Dialects acquired from the TopicExpressionEngine
     	 *
     	 * If this is not the behavior you would like, you will need to remove the following line of code and initialize the properties
     	 * individually.
         */
	    TopicSpaceSet aSet = getTopicSpaceSet();

	    /*  MUWS TopicSpace is returned by MuwsUtils.addRelationshipTopics - This utility operation adds relationship topics support. I
	     *  nternaly the code :
	     *  TopicSpace muwsTopicSpace = new TopicSpaceImpl( MuwsConstants.NSURI_MUWS_PART2_TOPICS )
	     *  aSet.addTopicSpace( muwsTopicSpace ) is executed. We are adding more MUWS topics to this
	     * 	TopicSpace.
	    */
	    TopicSpace muwsTopicSpace = MuwsUtils.addRelationshipTopics( aSet );

        ManagementEventTopic identityCapability = new XmlBeansManagementEventTopicImpl( IdentityCapability.TOPIC_NAME );
        muwsTopicSpace.addTopic( identityCapability );

        ManagementEventTopic manageabilityCharacteristicsCapability = new XmlBeansManagementEventTopicImpl( ManageabilityCharacteristicsCapability.TOPIC_NAME );
        muwsTopicSpace.addTopic( manageabilityCharacteristicsCapability );

        ManagementEventTopic operationalStatusCapability = new XmlBeansManagementEventTopicImpl(OperationalStatusCapability.TOPIC_NAME );
        muwsTopicSpace.addTopic( operationalStatusCapability );

        ManagementEventTopic muwsDescriptionCapability = new XmlBeansManagementEventTopicImpl( DescriptionCapability.TOPIC_NAME );
        muwsTopicSpace.addTopic( muwsDescriptionCapability );

        ManagementEventTopic metricCapability = new XmlBeansManagementEventTopicImpl(MetricsCapability.TOPIC_NAME );
        muwsTopicSpace.addTopic( metricCapability );


        //MOWS TopicSpace
        TopicSpace mowsTopicSpace = new TopicSpaceImpl( MowsConstants.NSURI_MOWS_TOPICS );
        getTopicSpaceSet().addTopicSpace( mowsTopicSpace );

        ManagementEventTopic mowsMetricsCapability = new XmlBeansManagementEventTopicImpl( MetricsCapability.TOPIC_NAME );
        mowsTopicSpace.addTopic( mowsMetricsCapability );


		// init the {http://ws.apache.org/muse/example/integrationserver/properties}ListenPort Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.LISTENPORT);
		org.apache.ws.muse.example.integrationserver.properties.ListenPortDocument prop_listenport = org.apache.ws.muse.example.integrationserver.properties.ListenPortDocument.Factory.newInstance();
		prop_listenport.setListenPort(getPort());
		resourceProperty.add(prop_listenport);


		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd}ResourceId Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.RESOURCEID);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ResourceIdDocument prop_resourceid = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ResourceIdDocument.Factory.newInstance();
		prop_resourceid.setResourceId(getID().toString());
		resourceProperty.add(prop_resourceid);
		resourceProperty.addChangeListener( identityCapability );


		// init the {http://org.apache.ws.namespaces.muse.muwsExt1}ResourceHostname Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.RESOURCEHOSTNAME);
		org.apache.ws.namespaces.muse.muwsExt1.ResourceHostnameDocument prop_resourcehostname = org.apache.ws.namespaces.muse.muwsExt1.ResourceHostnameDocument.Factory.newInstance();
		prop_resourcehostname.setResourceHostname(getHostName());
		resourceProperty.add(prop_resourcehostname);


		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}Version Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.VERSION);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.VersionDocument prop_version = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.VersionDocument.Factory.newInstance();
		prop_version.setVersion(VERSION);
		resourceProperty.add(prop_version);
		resourceProperty.addChangeListener(muwsDescriptionCapability);


		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}Caption Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.CAPTION);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CaptionDocument prop_caption = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CaptionDocument.Factory.newInstance();
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString lStringC = LangString.Factory.newInstance();
		lStringC.setStringValue(CAPTION);
		lStringC.setLang("en");
		prop_caption.setCaption(lStringC);
		resourceProperty.add(prop_caption);
		resourceProperty.addChangeListener(muwsDescriptionCapability);


		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd}ManageabilityCapability Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.MANAGEABILITYCAPABILITY);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument prop_manageabilitycapability = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument.Factory.newInstance();
		prop_manageabilitycapability.setManageabilityCapability( IdentityCapability.URI );
		resourceProperty.add(prop_manageabilitycapability);

		resourceProperty.addChangeListener( manageabilityCharacteristicsCapability );
		prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance();
		prop_manageabilitycapability.setManageabilityCapability( org.apache.ws.muws.v1_0.capability.MetricsCapability.URI );
        resourceProperty.add( prop_manageabilitycapability );

        prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance();
        prop_manageabilitycapability.setManageabilityCapability( RelationshipsCapability.URI );
        resourceProperty.add( prop_manageabilitycapability );

        prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance();
        prop_manageabilitycapability.setManageabilityCapability( DescriptionCapability.URI );
        resourceProperty.add( prop_manageabilitycapability );

        prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance();
        prop_manageabilitycapability.setManageabilityCapability(  OperationalStatusCapability.URI);
        resourceProperty.add( prop_manageabilitycapability );

        prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance();
		prop_manageabilitycapability.setManageabilityCapability( MetricsCapability.URI );
		resourceProperty.add(prop_manageabilitycapability);

        prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance();
        prop_manageabilitycapability.setManageabilityCapability(  ExampleConstants.MUWS_EXT_CAPABILITY_URI);
        resourceProperty.add( prop_manageabilitycapability );

        resourceProperty.addChangeListener( manageabilityCharacteristicsCapability );

		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}Description Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.DESCRIPTION);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.DescriptionDocument prop_description = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.DescriptionDocument.Factory.newInstance();
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString lString = LangString.Factory.newInstance();
		lString.setStringValue(DESCRIPTION);
		lString.setLang("en");
		prop_description.setDescription(lString);
		resourceProperty.add(prop_description);
		resourceProperty.addChangeListener(muwsDescriptionCapability);


		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}OperationalStatus Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.OPERATIONALSTATUS);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument prop_operationalstatus = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument.Factory.newInstance();
		prop_operationalstatus.setOperationalStatus(OperationalStatusDocument.OperationalStatus.AVAILABLE);
		resourceProperty.add(prop_operationalstatus);
		resourceProperty.addChangeListener(operationalStatusCapability);



		// init the {http://org.apache.ws.namespaces.muse.muwsExt1}ResourceType Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.RESOURCETYPE);
		org.apache.ws.namespaces.muse.muwsExt1.ResourceTypeDocument prop_resourcetype = org.apache.ws.namespaces.muse.muwsExt1.ResourceTypeDocument.Factory.newInstance();
		prop_resourcetype.setResourceType(RESOURCE_TYPE);
		resourceProperty.add(prop_resourcetype);


		// init the {http://ws.apache.org/muse/example/integrationserver/properties}ServerName Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.SERVERNAME);
		org.apache.ws.muse.example.integrationserver.properties.ServerNameDocument prop_servername = org.apache.ws.muse.example.integrationserver.properties.ServerNameDocument.Factory.newInstance();
		prop_servername.setServerName(CAPTION + " - " + getHostName());
		resourceProperty.add(prop_servername);

		// init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}CurrentTime ResourceProperty
        resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.CURRENTTIME);
        CurrentTimeDocument currentTimeDocument = CurrentTimeDocument.Factory.newInstance();
        currentTimeDocument.setCurrentTime(Calendar.getInstance());
        resourceProperty.add(currentTimeDocument);
        resourceProperty.setCallback(new org.apache.ws.resource.lifetime.callback.CurrentTimeCallback());
        resourceProperty.addChangeListener(metricCapability);

        // init the {http://ws.apache.org/namespaces/muse/muws-ext-1.xsd}Owner Resource Property
        resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.OWNER);
		org.apache.ws.namespaces.muse.muwsExt1.OwnerDocument prop_owner = org.apache.ws.namespaces.muse.muwsExt1.OwnerDocument.Factory.newInstance();
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString oString =  LangString.Factory.newInstance(  );
		oString.setStringValue( ExampleConstants.INT_SERVER_OWNER );
		oString.setLang( "en" );
        prop_owner.setOwner( oString );
		resourceProperty.add(prop_owner);

		// init the {http://ws.apache.org/namespaces/muse/muws-ext-1.xsd}CreationTime Resource Property
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.CREATIONTIME);
		org.apache.ws.namespaces.muse.muwsExt1.CreationTimeDocument prop_creationtime = org.apache.ws.namespaces.muse.muwsExt1.CreationTimeDocument.Factory.newInstance();
		prop_creationtime.setCreationTime(Calendar.getInstance(  ));
		resourceProperty.add(prop_creationtime);

		}
	catch (Exception e)
	{
	   throw new javax.xml.rpc.JAXRPCException("There was a problem in initializing your resource properties.  Please check your init() method. Cause: " + e.getLocalizedMessage());
	}

    }

    protected String getHostName()
	{
		String name = null;
		try
		{
			java.net.InetAddress localMachine =	java.net.InetAddress.getLocalHost();
			name = localMachine.getCanonicalHostName(  );
		}
		catch(java.net.UnknownHostException uhe)
		{
			LOG.error("Error getting HostName");
		}

		return name;
	}

    protected long getPort() throws  Exception
    {
    	long portNum = 8080L;
    	return portNum;
    }


    public void setRelationship(org.apache.ws.muse.example.resourceadmin.properties.SetRelationshipDocument requestDoc )throws Exception
    {
    	org.apache.ws.resource.properties.ResourcePropertySet resourcePropertySet = getResourcePropertySet();
		org.apache.ws.resource.properties.ResourceProperty resourceProperty = null;
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.RELATIONSHIP);
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipDocument prop_relationship = org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipDocument.Factory.newInstance();
		RelationshipType relationshipType = prop_relationship.addNewRelationship();
		relationshipType.setType(requestDoc.getSetRelationship().getType());

		RelationshipParticipantType [] partTo = requestDoc.getSetRelationship().getRelationshipTo().getParticipantArray();
		int numOfToParticipants = partTo.length;
		if(numOfToParticipants > 0)
		{
			//There is the from participant - will be in the first position in the array,
			// there needs to be at least one participant to, can be more.
			RelationshipParticipantType [] participants = new RelationshipParticipantType[numOfToParticipants + 1];
			participants[0] = requestDoc.getSetRelationship().getRelationshipFrom().getParticipant();
			for( int i=0; i<numOfToParticipants; i++)
				participants[i+1 ] = partTo[i];
		    relationshipType.setParticipantArray( participants);
			resourceProperty.add(prop_relationship);
			if ( LOG.isDebugEnabled() )
	         {
	             LOG.debug( "SetRelationship -   " + prop_relationship );
	         }
		}
		else
			throw new Exception ("Missing Relationship Participant");

    }

    public void removeRelationship(org.apache.ws.muse.example.resourceadmin.properties.RemoveRelationshipDocument requestDoc )throws Exception
    {
    	// get participants of relationship to be removed
    	RelationshipType relationshipRemove = requestDoc.getRemoveRelationship().getRelationship();

		// get the relationship property from properties set
		// then walk all relationships and compare them with the relationship which needs to be removed.
		// when the match exist the relationship will be removed from the relationship properties list
		// this will genarate realtionshipremoved notification
    	org.apache.ws.resource.properties.ResourcePropertySet resourcePropertySet = getResourcePropertySet();
		org.apache.ws.resource.properties.ResourceProperty resourceProperty = null;
		resourceProperty = resourcePropertySet.get(IntegrationserverPropertyQNames.RELATIONSHIP);


		if (resourceProperty.isEmpty())
			return;
		for ( int i=0; i < resourceProperty.size(); i++)
		{
			Object relationObj = resourceProperty.get(i);
			if ( relationObj instanceof RelationshipType)
			{
				RelationshipType relationship = (RelationshipType)relationObj;
				if( ExampleUtils.CompareRelationships(relationshipRemove, relationship ))
				{
					resourceProperty.remove(relationObj);
					//continue to walk - you can have more then one this same relationship
					//the notification will be sent to wsdm:RelationshipsCapability/RelationshipDeleted topic
				}

			}
		}
 }



}
