/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muse.example.resourceadmin;

import org.apache.ws.mows.v1_0.capability.OperationalStatusCapability;
import org.apache.ws.muse.example.ExampleConstants;
import org.apache.ws.muse.example.resourceadmin.properties.DiscoveryType;
import org.apache.ws.muse.example.resourceadmin.properties.SupportedResourceType;
import org.apache.ws.muws.v1_0.MuwsConstants;
import org.apache.ws.muws.v1_0.capability.AdvertisementCapability;
import org.apache.ws.muws.v1_0.capability.DescriptionCapability;
import org.apache.ws.muws.v1_0.capability.IdentityCapability;
import org.apache.ws.muws.v1_0.capability.ManageabilityCharacteristicsCapability;
import org.apache.ws.muws.v1_0.capability.MetricsCapability;
import org.apache.ws.muws.v1_0.impl.advertiser.ResourceAdvertiserPropertyQNames;
import org.apache.ws.muws.v1_0.topics.ManagementEventTopic;
import org.apache.ws.muws.v1_0.topics.impl.XmlBeansAdvertisementTopicImpl;
import org.apache.ws.muws.v1_0.topics.impl.XmlBeansManagementEventTopicImpl;
import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicSpace;
import org.apache.ws.notification.topics.impl.TopicSpaceImpl;
import org.apache.ws.resource.impl.AbstractResourceHome;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CurrentTimeDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument;
import org.xmlsoap.schemas.ws.x2004.x08.addressing.EndpointReferenceType;
import javax.naming.InitialContext;
import java.util.Calendar;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author Kinga Dziembowski
 *
 * A Resourceadmin WS-Resource.
 * <p/>
 * NOTE: This class is generated but IS meant to be modified.
 */
public class ResourceadminResource
   extends AbstractResourceadminResource
{
   /**
    * A NamespaceVerionHolder which maintains the QNames of Spec Wsdls
    */
   public static final org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl SPEC_NAMESPACE_SET =
      new org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl(  );

   /**
    * A constant resource Id for singleton
    */
   public static final String ADMIN_RESOURCE_ID = "Resource-Admin";

   /**
    * A ResourceAdnin resource version
    */
   public static final String VERSION = "1.0";

   /**
    * A ResourceAdnin resource caption
    */
   public static final String CAPTION = "RESOURCE-ADMIN";

   /**
    * A ResourceAdnin resource Resource Type
    */
   public static final String RESOURCE_TYPE = "AdminForResources";

   /** A log object */
   private static final Log LOG = LogFactory.getLog( ResourceadminResource.class.getName() );


   /**
    * Initializes this resource's state (properties, etc.).
    */
   public void init(  )
   {
      super.init(  );

      /**
       * The ResourcePropertySet which contains all the defined ResourceProperties
       */
      org.apache.ws.resource.properties.ResourcePropertySet resourcePropertySet = getResourcePropertySet(  );
      org.apache.ws.resource.properties.ResourceProperty    resourceProperty = null;

      try
      {
         // init the {http://org.apache.ws.muse.example.resourceadmin.properties}SupportedResources Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.SUPPORTEDRESOURCES );
         org.apache.ws.muse.example.resourceadmin.properties.SupportedResourcesDocument prop_supportedresources =
            org.apache.ws.muse.example.resourceadmin.properties.SupportedResourcesDocument.Factory.newInstance(  );
         SupportedResourceType                                                          supportedTypes =
            SupportedResourceType.Factory.newInstance(  );

         String[]                                                                       resourceTypes =
            ExampleConstants.getKnownResourceTypes(  );
         for ( int i = 0; i < resourceTypes.length; i++ )
         {
            supportedTypes.addNewResourceType(  ).setStringValue( resourceTypes[i] );
         }

         prop_supportedresources.setSupportedResources( supportedTypes );
         resourceProperty.add( prop_supportedresources );

         // Topics
         org.apache.ws.notification.topics.TopicSpaceSet topicSpaceSet  = getTopicSpaceSet(  );
         TopicSpace                                      muwsTopicSpace =
            new TopicSpaceImpl( MuwsConstants.NSURI_MUWS_PART2_TOPICS );

         Topic                                           manageabilityEndpointCreation    =
            new XmlBeansAdvertisementTopicImpl( AdvertisementCapability.TOPIC_MANAGEABILITY_ENPOINT_CREATION_NAME,
                                                false );
         Topic                                           manageabilityEndpointDestruction =
            new XmlBeansAdvertisementTopicImpl( AdvertisementCapability.TOPIC_MANAGEABILITY_ENPOINT_DESTRUCTION_NAME,
                                                false );

         XmlBeansAdvertisementTopicImpl                  manageableResourceCreation    =
            new XmlBeansAdvertisementTopicImpl( AdvertisementCapability.TOPIC_MANAGEABLE_RESOURCE_CREATION_NAME );
         XmlBeansAdvertisementTopicImpl                  manageableResourceDestruction =
            new XmlBeansAdvertisementTopicImpl( AdvertisementCapability.TOPIC_MANAGEABLE_RESOURCE_DESTRUCTION_NAME );

         //register listeners for those topics with all known to resourceadmin  homes....
         addListenersToHomes( manageableResourceCreation, manageableResourceDestruction );

         manageabilityEndpointCreation.addTopic( manageableResourceCreation );
         manageabilityEndpointDestruction.addTopic( manageableResourceDestruction );
         muwsTopicSpace.addTopic( manageabilityEndpointCreation );
         muwsTopicSpace.addTopic( manageabilityEndpointDestruction );
         topicSpaceSet.addTopicSpace( muwsTopicSpace );

         ManagementEventTopic identityCapability =
            new XmlBeansManagementEventTopicImpl( IdentityCapability.TOPIC_NAME );
         muwsTopicSpace.addTopic( identityCapability );
         ManagementEventTopic manageabilityCharacteristicsCapability =
            new XmlBeansManagementEventTopicImpl( ManageabilityCharacteristicsCapability.TOPIC_NAME );
         muwsTopicSpace.addTopic( manageabilityCharacteristicsCapability );
         ManagementEventTopic operationalStatusCapability =
            new XmlBeansManagementEventTopicImpl( OperationalStatusCapability.TOPIC_NAME );
         muwsTopicSpace.addTopic( operationalStatusCapability );
         ManagementEventTopic muwsDescriptionCapability =
            new XmlBeansManagementEventTopicImpl( DescriptionCapability.TOPIC_NAME );
         muwsTopicSpace.addTopic( muwsDescriptionCapability );
         ManagementEventTopic metricCapability =
            new XmlBeansManagementEventTopicImpl( MetricsCapability.TOPIC_NAME );
         muwsTopicSpace.addTopic( metricCapability );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd}ResourceId Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.RESOURCEID );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ResourceIdDocument prop_resourceid =
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ResourceIdDocument.Factory.newInstance(  );
         prop_resourceid.setResourceId( ADMIN_RESOURCE_ID );
         resourceProperty.add( prop_resourceid );

         // init the {http://org.apache.ws.namespaces.muse.muwsExt1}ResourceHostname Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.RESOURCEHOSTNAME );
         org.apache.ws.namespaces.muse.muwsExt1.ResourceHostnameDocument prop_resourcehostname =
            org.apache.ws.namespaces.muse.muwsExt1.ResourceHostnameDocument.Factory.newInstance(  );
         prop_resourcehostname.setResourceHostname( getHostName(  ) );
         resourceProperty.add( prop_resourcehostname );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}Version Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.VERSION );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.VersionDocument prop_version =
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.VersionDocument.Factory.newInstance(  );
         prop_version.setVersion( VERSION );
         resourceProperty.add( prop_version );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd}ManageabilityCapability Resource Property
         resourceProperty = resourcePropertySet.get( ResourceAdvertiserPropertyQNames.MANAGEABILITYCAPABILITY );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument prop_manageabilitycapability =
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument.Factory
            .newInstance(  );
         prop_manageabilitycapability.setManageabilityCapability( AdvertisementCapability.URI );
         resourceProperty.add( prop_manageabilitycapability );

         prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance(  );
         prop_manageabilitycapability.setManageabilityCapability( IdentityCapability.URI );
         resourceProperty.add( prop_manageabilitycapability );

         prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance(  );
         prop_manageabilitycapability.setManageabilityCapability( DescriptionCapability.URI );
         resourceProperty.add( prop_manageabilitycapability );

         prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance(  );
         prop_manageabilitycapability.setManageabilityCapability( MetricsCapability.URI );
         resourceProperty.add( prop_manageabilitycapability );

         prop_manageabilitycapability = ManageabilityCapabilityDocument.Factory.newInstance(  );
         prop_manageabilitycapability.setManageabilityCapability( ExampleConstants.MUWS_EXT_CAPABILITY_URI );
         resourceProperty.add( prop_manageabilitycapability );

         resourceProperty.addChangeListener( manageabilityCharacteristicsCapability );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}Caption Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.CAPTION );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CaptionDocument prop_caption =
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CaptionDocument.Factory.newInstance(  );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString      lStringC =
            LangString.Factory.newInstance(  );
         lStringC.setStringValue( CAPTION );
         lStringC.setLang( "en" );
         prop_caption.setCaption( lStringC );
         resourceProperty.add( prop_caption );

         // init the {http://ws.apache.org/muse/example/resourceadmin/properties}Discovery Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.DISCOVERY );
         org.apache.ws.muse.example.resourceadmin.properties.DiscoveryDocument prop_discovery =
            org.apache.ws.muse.example.resourceadmin.properties.DiscoveryDocument.Factory.newInstance(  );
         DiscoveryType discType = prop_discovery.addNewDiscovery(  );

         // During the initialisation the list of EPR is empty
         EndpointReferenceType[] eprs = new EndpointReferenceType[0];
         discType.setEndpointReferenceArray( eprs );
         resourceProperty.setCallback( new DiscoveryCallback( this ) );
         resourceProperty.add( prop_discovery );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}Description Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.DESCRIPTION );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.DescriptionDocument prop_description =
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.DescriptionDocument.Factory.newInstance(  );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString          lString =    LangString.Factory.newInstance(  );
         lString.setStringValue( "This is an Admin for resources objects" );
         lString.setLang( "en" );
         prop_description.setDescription( lString );
         resourceProperty.add( prop_description );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}OperationalStatus Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.OPERATIONALSTATUS );
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument prop_operationalstatus =
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument.Factory.newInstance(  );
         prop_operationalstatus.setOperationalStatus( OperationalStatusDocument.OperationalStatus.AVAILABLE );
         resourceProperty.add( prop_operationalstatus );

         // init the {http://org.apache.ws.namespaces.muse.muwsExt1}ResourceType Resource Property
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.RESOURCETYPE );
         org.apache.ws.namespaces.muse.muwsExt1.ResourceTypeDocument prop_resourcetype =
            org.apache.ws.namespaces.muse.muwsExt1.ResourceTypeDocument.Factory.newInstance(  );
         prop_resourcetype.setResourceType( RESOURCE_TYPE );
         resourceProperty.add( prop_resourcetype );

         // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd}CurrentTime ResourceProperty
         resourceProperty = resourcePropertySet.get( ResourceadminPropertyQNames.CURRENTTIME );
         CurrentTimeDocument currentTimeDocument = CurrentTimeDocument.Factory.newInstance(  );
         currentTimeDocument.setCurrentTime( Calendar.getInstance(  ) );
         resourceProperty.add( currentTimeDocument );
         resourceProperty.setCallback( new org.apache.ws.resource.lifetime.callback.CurrentTimeCallback(  ) );
         resourceProperty.addChangeListener( metricCapability );

         // init the {http://ws.apache.org/namespaces/muse/muws-ext-1.xsd}Owner Resource Property
		resourceProperty = resourcePropertySet.get(ResourceadminPropertyQNames.OWNER);
		org.apache.ws.namespaces.muse.muwsExt1.OwnerDocument prop_owner = org.apache.ws.namespaces.muse.muwsExt1.OwnerDocument.Factory.newInstance();
		org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString oString =  LangString.Factory.newInstance(  );
		oString.setStringValue( ExampleConstants.RES_ADMIN_OWNER );
		oString.setLang( "en" );
        prop_owner.setOwner( oString );
		resourceProperty.add(prop_owner);

		// init the {http://ws.apache.org/namespaces/muse/muws-ext-1.xsd}CreationTime Resource Property
		resourceProperty = resourcePropertySet.get(ResourceadminPropertyQNames.CREATIONTIME);
		org.apache.ws.namespaces.muse.muwsExt1.CreationTimeDocument prop_creationtime = org.apache.ws.namespaces.muse.muwsExt1.CreationTimeDocument.Factory.newInstance();
		prop_creationtime.setCreationTime(Calendar.getInstance(  ));
		resourceProperty.add(prop_creationtime);

      }
      catch ( Exception e )
      {
         throw new javax.xml.rpc.JAXRPCException( "There was a problem in initializing your resource properties.  Please check your init() method. Cause: "
                                                  + e.getLocalizedMessage(  ) );
      }

      // Resource Property {http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd}FixedTopicSet is implemented by the framework.
      // Resource Property {http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd}Topic is implemented by the framework.
      // Resource Property {http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd}TopicExpressionDialects is implemented by the framework.

   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected String getHostName(  )
   {
      String name = null;

      try
      {
         java.net.InetAddress localMachine = java.net.InetAddress.getLocalHost(  );
         name = localMachine.getCanonicalHostName(  );

      }
      catch ( java.net.UnknownHostException uhe )
      {
         System.out.println( "Error getting HostName" );
      }

      return name;
   }

   /**
    * Connects the homes for the resources withe the topics for creation/destruction of resources.
    * This way when the resource is created/destroyed the AdvertisementCapability
    * notifications will be sent to subscribers.
    *
    * @param manageableResourceCreation
    * @param manageableResourceDestruction
    * @throws Exception
    */
   protected void addListenersToHomes( XmlBeansAdvertisementTopicImpl manageableResourceCreation,
                                       XmlBeansAdvertisementTopicImpl manageableResourceDestruction )
   throws Exception
   {
      InitialContext    ctx   = new InitialContext(  );
      String[] homes = ResourceadminHome.getKnownHomes();
      for ( int i = 0; i < homes.length; i++ )
      {
         AbstractResourceHome absHome = (AbstractResourceHome) ctx.lookup( homes[i] );
         absHome.addResourceCreationListener( manageableResourceCreation );
         absHome.addResourceDestructionListener( manageableResourceDestruction );
      }
   }
}