package example.filesystem.callback;

import example.filesystem.backend.FileSystem;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.ResourcePropertyCallback;
import org.apache.ws.resource.properties.impl.CallbackFailedException;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument;

/**
 * A callback for the MUWS OperationalStatus resource property.
 */
public class OperationalStatusCallback implements ResourcePropertyCallback
{

    private FileSystem m_fileSystem;

    public OperationalStatusCallback(FileSystem fileSystem)
    {
        m_fileSystem = fileSystem;
    }

    public ResourceProperty refreshProperty(ResourceProperty prop)  throws CallbackFailedException
    {
        if ( prop.isEmpty() )
        {
            OperationalStatusDocument statusDoc = OperationalStatusDocument.Factory.newInstance();
            statusDoc.setOperationalStatus( getBackendStatus() );
            prop.add( statusDoc );
        }
        else
        {
            OperationalStatusDocument.OperationalStatus status = (OperationalStatusDocument.OperationalStatus) prop.get( 0 );
            status.set( getBackendStatus() );
        }
        return prop;
    }

    private OperationalStatusDocument.OperationalStatus.Enum getBackendStatus()
    {
        return m_fileSystem.isMounted() ? OperationalStatusDocument.OperationalStatus.AVAILABLE : OperationalStatusDocument.OperationalStatus.UNAVAILABLE;
    }
}
