package org.wsdmdemo.service;

import org.wsdmdemo.service.weatherStationDir.WeatherStationDirHome;

import javax.xml.namespace.QName;

/**
 * Configuration Constants for the Demo Application and Unit Test
 *
 * @author Sal Campana
 */
public interface InteropConstants
{

    /**
     * Sysprops for config params
     */
    String SYSPROP_WS1_EPR_URL = "WS1_EPR_URL";
    String SYSPROP_WS_DIR_SERVICE_URL = "WS_DIR_SERVICE_URL";
    String SYSPROP_WS_CLIENT_CONFIG_EPR_URL = "WS_CLIENT_CONFIG_EPR_URL";
    String SYSPROP_WS2_EPR_URL = "WS2_EPR_URL";
    String SYSPROP_BASE_ADDRESS = "BASE_ADDRESS";
    String SYSPROP_REFRESH_INTERVAL = "REFRESH";

    /**
     * Resource Keys for the Weather Stations *
     */
    String WS1_KEY = "weather-station-1";
    String WS2_KEY = "weather-station-2";

    /**
     * default relationship *
     */
    QName RELATIONSHIP_RELATION = new QName(WeatherStationDirHome.SERVICE_NAME.getNamespaceURI(), "Contains", "wsdir");
    String PARENT_RELATION = "urn://parent";
    String CHILD_RELATION = "urn://child";

    /**
     * Directory service Muws ResourceID *
     */
    String DIR_RESOURCEID = "urn:SINGLETON";


    /**
     * Min/Max Values
     */
    double MAX_HUMIDITY = 70.0;
    double MIN_HUMIDITY = 60.0;
    double MAX_VISIBILITY = 11.0;
    double MIN_VISIBILITY = 9.0;
    int MAX_TEMP = 78;
    int MIN_TEMP = 75;

    /**
     * Default Values
     */
    double DEFAULT_HUMIDITY = 65.0;
    double DEFAULT_TEMP = 69.0;
    double DEFAULT_VISIBILITY = 10.09;
    double PRICE = 0.99;
    String OWNER = "Weather.com";

    /**
     * Refresh Interval for retrieving data...initially 20 seconds.
     */
    int INITIAL_REFRESH = Integer.parseInt(System.getProperty(SYSPROP_REFRESH_INTERVAL, "10"));


    /**
     * EPR URLS
     */
    String WEATHER_STATION_1_EPR_URL = System.getProperty(InteropConstants.SYSPROP_WS1_EPR_URL, "http://localhost:8080/muse/epr/weather-station-1-epr.xml");
    String WEATHER_STATION_2_EPR_URL = System.getProperty(InteropConstants.SYSPROP_WS2_EPR_URL, "http://localhost:8080/muse/epr/weather-station-2-epr.xml");

    /**
     * BASE SERVICE ADDRESS *
     */
    String BASE_ADDRESS = System.getProperty(SYSPROP_BASE_ADDRESS, "http://localhost:8080/muse/services");

    /**
     * Interval for changing operational status back to available
     */
    int OPERATIONAL_STATUS_RESET_INTERVAL = Integer.parseInt(System.getProperty("OPERATIONAL_STATUS_RESET_INTERVAL","20000"));
}
