
import junit.framework.TestCase;
import org.apache.ws.addressing.XmlBeansEndpointReference;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.GetResourcePropertyDocument;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.SetResourcePropertiesDocument;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.UpdateType;
import org.wsdmdemo.service.InteropConstants;
import org.wsdmdemo.service.InteropRequestUtils;
import org.wsdmdemo.service.weatherClientConfig.WeatherClientConfigPropertyQNames;
import org.wsdmdemo.service.weatherClientConfig.WeatherStationReferenceDocument;
import org.wsdmdemo.service.weatherStation.RecalibrateDocument;
import org.wsdmdemo.service.weatherStation.WeatherStationPropertyQNames;
import org.wsdmdemo.service.weatherStationDir.AddWeatherStationDocument;
import org.wsdmdemo.service.weatherStationDir.WeatherStationDirPropertyQNames;
import org.xmlsoap.schemas.ws.x2004.x08.addressing.EndpointReferenceDocument;

import java.net.URL;

/**
 * @author Sal Campana
 */
public class IEEEDemoTest extends TestCase
{
    private static final String WEATHER_STATION_1_EPR_URL = System.getProperty(InteropConstants.SYSPROP_WS1_EPR_URL, "http://localhost:8080/muse/epr/weather-station-1-epr.xml");
    private static final String WEATHER_STATION_2_EPR_URL = System.getProperty(InteropConstants.SYSPROP_WS2_EPR_URL, "http://localhost:8080/muse/epr/weather-station-2-epr.xml");
    private static final String WEATHER_STATION_DIR_SERVICE_URL = System.getProperty(InteropConstants.SYSPROP_WS_DIR_SERVICE_URL, "http://localhost:8080/muse/services/WeatherStationDir");
    private static final String WEATHER_STATION_CLIENT_CONFIG_EPR_URL = System.getProperty(InteropConstants.SYSPROP_WS_CLIENT_CONFIG_EPR_URL, "http://localhost:8080/muse/epr/weather-client-config-epr.xml");;
    private static EndpointReferenceDocument WEATHER_STATION_1_EPR_DOC;
    private static EndpointReferenceDocument WEATHER_STATION_2_EPR_DOC;
    private static EndpointReferenceDocument CLIENT_CONFIG_EPR;

    protected void setUp() throws Exception
    {
        WEATHER_STATION_1_EPR_DOC = InteropRequestUtils.getEndpointReference(new URL(WEATHER_STATION_1_EPR_URL));
        WEATHER_STATION_2_EPR_DOC = InteropRequestUtils.getEndpointReference(new URL(WEATHER_STATION_2_EPR_URL));
        CLIENT_CONFIG_EPR = InteropRequestUtils.getEndpointReference(new URL(WEATHER_STATION_CLIENT_CONFIG_EPR_URL));
    }

    public void testGetConfiguredWeatherStation()
    {
        EndpointReferenceDocument clientConfigEpr = CLIENT_CONFIG_EPR;

        //retrieve the epr for weather station 1 from wc
        GetResourcePropertyDocument getResourcePropertyDocument = GetResourcePropertyDocument.Factory.newInstance();
        getResourcePropertyDocument.setGetResourceProperty(WeatherClientConfigPropertyQNames.WEATHERSTATIONREFERENCE);
        XmlObject getResp = InteropRequestUtils.sendRequest(getResourcePropertyDocument, "http://wsdmdemo.org/service/weather-station-dir/GetResourceProperties", new XmlBeansEndpointReference(clientConfigEpr.getEndpointReference()));

    }

    public void testDemo() throws Exception
    {
        EndpointReferenceDocument weatherStationEprDoc = WEATHER_STATION_1_EPR_DOC;
        EndpointReferenceDocument clientConfigEpr = CLIENT_CONFIG_EPR;

        //retrieve the epr for weather station 1 from wc
        GetResourcePropertyDocument getResourcePropertyDocument = GetResourcePropertyDocument.Factory.newInstance();
        getResourcePropertyDocument.setGetResourceProperty(WeatherClientConfigPropertyQNames.WEATHERSTATIONREFERENCE);
        XmlObject getResp = InteropRequestUtils.sendRequest(getResourcePropertyDocument, "http://wsdmdemo.org/service/weather-station-dir/GetResourceProperties", new XmlBeansEndpointReference(clientConfigEpr.getEndpointReference()));

        //assertTrue(getResp instanceof )
        //todo check its ws1

        getResourcePropertyDocument = GetResourcePropertyDocument.Factory.newInstance();
        getResourcePropertyDocument.setGetResourceProperty(WeatherStationPropertyQNames.TEMPERATURE);
        getResp = InteropRequestUtils.sendRequest(getResourcePropertyDocument, "http://wsdmdemo.org/service/weather-station-dir/GetResourceProperties", new XmlBeansEndpointReference(weatherStationEprDoc.getEndpointReference()));
        //todo assert

        /* //retrieve the metrics
         GetMultipleResourcePropertiesDocument getMultipleResourcePropertiesDocument = GetMultipleResourcePropertiesDocument.Factory.newInstance();
         GetMultipleResourcePropertiesDocument.GetMultipleResourceProperties getMultipleResourceProperties = getMultipleResourcePropertiesDocument.addNewGetMultipleResourceProperties();
         QName[] metrics = {WeatherStationPropertyQNames.HUMIDITY, WeatherStationPropertyQNames.TEMPERATURE, WeatherStationPropertyQNames.VISIBILITY};
         getMultipleResourceProperties.setResourcePropertyArray(metrics);
         XmlObject getMultResp = InteropRequestUtils.sendRequest(getMultipleResourcePropertiesDocument,"http://wsdmdemo.org/service/weather-station-dir/GetMultipleResourceProperties",new XmlBeansEndpointReference(weatherStationEprDoc.addNewEndpointReference()));

         //pull again should be different
         XmlObject getMultResp2 = InteropRequestUtils.sendRequest(getMultipleResourcePropertiesDocument,"http://wsdmdemo.org/service/weather-station-dir/GetMultipleResourceProperties",new XmlBeansEndpointReference(weatherStationEprDoc.addNewEndpointReference()));
         */

        testgetRelationships();

        //check the OpStatus prop is PartiallyAvail .....also need to check notif
        getResourcePropertyDocument = GetResourcePropertyDocument.Factory.newInstance();
        getResourcePropertyDocument.setGetResourceProperty(WeatherStationPropertyQNames.OPERATIONALSTATUS);
        getResp = InteropRequestUtils.sendRequest(getResourcePropertyDocument, "http://wsdmdemo.org/service/weather-station-dir/GetResourceProperties", new XmlBeansEndpointReference(weatherStationEprDoc.getEndpointReference()));
        //todo check avail

        //recalibrate ws1
        RecalibrateDocument recalibrateDocument = RecalibrateDocument.Factory.newInstance();
        org.wsdmdemo.service.weatherStation.RecalibrateDocument.Recalibrate recalibrate = recalibrateDocument.addNewRecalibrate();
        XmlObject recalibResponse = InteropRequestUtils.sendRequest(recalibrateDocument, "http://wsdmdemo.org/service/weather-station-dir/Recalibrate", new XmlBeansEndpointReference(weatherStationEprDoc.getEndpointReference()));
        //todo assert not fault

        //check the OpStatus prop is PartiallyAvail .....also need to check notif
        getResourcePropertyDocument = GetResourcePropertyDocument.Factory.newInstance();
        getResourcePropertyDocument.setGetResourceProperty(WeatherStationPropertyQNames.OPERATIONALSTATUS);
        getResp = InteropRequestUtils.sendRequest(getResourcePropertyDocument, "http://wsdmdemo.org/service/weather-station-dir/GetResourceProperties", new XmlBeansEndpointReference(weatherStationEprDoc.getEndpointReference()));
        //todo check unavail


        //todo test notifs....???are there notifs anymore??
    }

    public void testgetRelationships()
    {
        GetResourcePropertyDocument getResourcePropertyDocument;
        //get relationships from dir > 0
        getResourcePropertyDocument = GetResourcePropertyDocument.Factory.newInstance();
        getResourcePropertyDocument.setGetResourceProperty(WeatherStationDirPropertyQNames.RELATIONSHIP);     //need epr
        XmlObject getRelResp = InteropRequestUtils.sendRequest(getResourcePropertyDocument, "http://wsdmdemo.org/service/weather-station-dir/GetResourceProperties", WEATHER_STATION_DIR_SERVICE_URL);
        //todo assert we get both relationships
    }

    public void testSetWeatherStation() throws Exception
    {
        EndpointReferenceDocument weatherStationEprDoc = WEATHER_STATION_2_EPR_DOC;
        EndpointReferenceDocument clientConfigEpr = CLIENT_CONFIG_EPR;
        SetResourcePropertiesDocument setResourcePropertiesDocument = SetResourcePropertiesDocument.Factory.newInstance();
        SetResourcePropertiesDocument.SetResourceProperties setResourceProperties = setResourcePropertiesDocument.addNewSetResourceProperties();
        UpdateType update = UpdateType.Factory.newInstance();
        WeatherStationReferenceDocument weatherStationReferenceDocument = WeatherStationReferenceDocument.Factory.newInstance();
        weatherStationReferenceDocument.setWeatherStationReference(weatherStationEprDoc.getEndpointReference());
        XmlBeanUtils.addChildElement(update, weatherStationReferenceDocument);
        setResourceProperties.setUpdateArray(new UpdateType[]{update});
        XmlObject addResponse = InteropRequestUtils.sendRequest(setResourcePropertiesDocument, "http://wsdmdemo.org/service/weather-station-dir/SetResourceProperties", new XmlBeansEndpointReference(clientConfigEpr.getEndpointReference()));

    }

    public void testAddWeatherStation()
    {
        EndpointReferenceDocument weatherStationEprDoc = WEATHER_STATION_2_EPR_DOC;
        //addweatherstation
        AddWeatherStationDocument addWeatherStationDocument = AddWeatherStationDocument.Factory.newInstance();
        org.wsdmdemo.service.weatherStationDir.AddWeatherStationDocument.AddWeatherStation addWeatherStation = addWeatherStationDocument.addNewAddWeatherStation();

        addWeatherStation.setEndpointReference(weatherStationEprDoc.getEndpointReference());
        XmlObject addResponse = InteropRequestUtils.sendRequest(addWeatherStationDocument, "http://wsdmdemo.org/service/weather-station-dir/AddWeatherStation", WEATHER_STATION_DIR_SERVICE_URL);
    }

   
}
