/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.interop.client;

import com.dell.wsdm.demos.interop.x2005.x04.AttentionLedOnDocument;
import com.dell.wsdm.demos.interop.x2005.x04.IpmiserverPropertyQNames;
import org.apache.ws.addressing.XmlBeansEndpointReference;
import org.apache.ws.muws.v1_0.capability.ConfigurationCapability;
import org.apache.ws.muws.v1_0.capability.ManageabilityCharacteristicsCapability;
import org.apache.ws.muws.v1_0.capability.OperationalStatusCapability;
import org.apache.xmlbeans.XmlBoolean;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.EndpointReferenceDocument;

import java.net.URL;

/**
 * A test case containing a test for each act in the "Server Management"
 * playbook, as defined in the WSDM 1.0 interop scenarios doc.
 *
 * @author Ian Springer
 */
public class ServerMgmtTestCase extends AbstractWsdmInteropTestCase
{

    public static final String SYSPROP_IPMI_SERVER_EPR = "ipmi_server.epr";

    private static final String DEFAULT_IPMI_SERVER_EPR = "http://cvs.apache.org/~ips/interop/ipmi_server-epr.xml";
    private static final String IPMI_SERVER_EPR = System.getProperty( SYSPROP_IPMI_SERVER_EPR, DEFAULT_IPMI_SERVER_EPR );

    private static final String NSURI_IPMI_SERVER = "http://dell.com/wsdm/demos/interop/2005/04";
    private static final String CAPABILITY_URI_SERVER_CONFIGURATION = NSURI_IPMI_SERVER + "/ServerConfiguration";
    private static final String CAPABILITY_URI_SERVER_CHASSIS_CONFIGURATION = NSURI_IPMI_SERVER + "/ServerChassisConfiguration";
    private static final String CAPABILITY_URI_SERVER_ALERT_STATUS = NSURI_IPMI_SERVER + "/ServerAlertStatus";

    private ResourceStub m_resource;

    protected void setUp() throws Exception
    {
        super.setUp();
        EndpointReferenceDocument ipmiServerEprDoc = getEndpointReference( new URL( IPMI_SERVER_EPR ) );
        if ( DEBUG ) { System.out.println( "*** Using IPMI Server EPR:\n" + ipmiServerEprDoc ); }
        m_resource = new ResourceStub( new XmlBeansEndpointReference(ipmiServerEprDoc.getEndpointReference()) );
    }

    public void testMuwsIdentityCharacteristics() throws FaultException
    {
        performMuwsIdentityCharacteristicsAct( m_resource );
    }

    public void testServerAssets() throws FaultException
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty(
                ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, ConfigurationCapability.URI );
        assertContainsURI( capabilityElems, CAPABILITY_URI_SERVER_CONFIGURATION );
        XmlObject serialNumberPropElem = getSingleProperty( m_resource, IpmiserverPropertyQNames.SERIALNUMBER );
        assertTrue( serialNumberPropElem instanceof XmlString );
        System.out.println( "Value of svr:SerialNumber property is: " + ((XmlString)serialNumberPropElem).getStringValue() );
        System.out.println( "*** Please verify that this result matches the known manufacturer's serial number." );
        XmlObject modelNamePropElem = getSingleProperty( m_resource, IpmiserverPropertyQNames.MODELNAME );
        assertTrue( modelNamePropElem instanceof XmlString );
        XmlObject ipNamePropElem = getSingleProperty( m_resource, IpmiserverPropertyQNames.IPNAME );
        assertTrue( ipNamePropElem instanceof XmlString );
        XmlObject sysContactPropElem = getSingleProperty( m_resource, IpmiserverPropertyQNames.SYSCONTACT );
        assertTrue( sysContactPropElem instanceof XmlString );
    }

    public void testServerAlert() throws FaultException
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty(
                ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, OperationalStatusCapability.URI );
        assertContainsURI( capabilityElems, CAPABILITY_URI_SERVER_ALERT_STATUS );
        AttentionLedOnDocument attentionLedOnDoc = AttentionLedOnDocument.Factory.newInstance();
        attentionLedOnDoc.setAttentionLedOn( true );
        setSingleProperty( m_resource, attentionLedOnDoc );
        System.out.println( "svr:AttentionLedOn property has been set to true." );
        System.out.println( "*** Please verify that the front panel attention LED blinks for approximately ten seconds." );
    }

    public void testServerChassisIntrusionDetection() throws FaultException
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty(
                ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, ConfigurationCapability.URI );
        assertContainsURI( capabilityElems, CAPABILITY_URI_SERVER_CHASSIS_CONFIGURATION );
        XmlObject coverOpenPropElem = getSingleProperty( m_resource, IpmiserverPropertyQNames.COVEROPEN );
        assertTrue( coverOpenPropElem instanceof XmlBoolean );
        System.out.println( "Value of svr:CoverOpen property is: " + ((XmlBoolean)coverOpenPropElem).getStringValue() );
        System.out.println( "*** Please verify that this result matches the current visible state of the front panel bezel and other interlocked covers of the server." );
    }

    public void testMuwsStatus() throws FaultException
    {
        performMuwsStatusAct( m_resource );
    }

}
