/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.xmlbeans;

import junit.framework.TestCase;
import org.apache.ws.util.JaxpUtils;
import org.apache.ws.util.XmlBeanUtils;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeDocument;
import org.w3c.dom.Node;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.AttributedURI;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.EndpointReferenceType;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.ReferencePropertiesType;

import javax.xml.namespace.QName;
import java.net.URL;

/**
 * TODO
 */
public class BugTester extends TestCase
{

    public void testParseMgmtEventXml() throws Exception
    {
        XmlObject xBean = XmlObject.Factory.parse(
                "<?xml version=\"1.0\" encoding=\"utf-8\"?><soap:Envelope xmlns:wsa=\"http://schemas.xmlsoap.org/ws/2003/03/addressing\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"><soap:Header><wsa:Action>http://docs.oasis-open.org/wsn/2004/06/WS-BaseNotification/Notify</wsa:Action><wsa:MessageID>uuid:9ef8b30f-0887-4e48-a922-b06fa6f5964b</wsa:MessageID><wsa:To>http://156.152.12.172:9101/</wsa:To></soap:Header><soap:Body><Notify xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns=\"http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd\"><NotificationMessage><Topic xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:wsdm=\"http://docs.oasis-open.org/wsdm/2004/12/mows/wsdm-mows-events.xml\" xmlns:wsn=\"http://docs.oasis-open.org/wsn/2004/06/wsn-WS-BaseNotification-1.2-draft-01.xsd\" Dialect=\"http://docs.oasis-open.org/wsn/2004/06/TopicExpression/Simple\">wsdm:RequestProcessingObservations</Topic><Message><ManagementEvent ReportTime=\"2005-04-13T13:34:09.6562816-04:00\" xmlns=\"http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd\"><EventId>eid:c2e0fc75-272a-4ac7-b08f-28de513e1219</EventId><SourceComponent><ResourceId>rid:e3a0b27a-45e0-40af-a15b-1cdc263f24ce</ResourceId></SourceComponent><RequestProcessingNotification CurrentTime=\"2005-04-13T13:34:09.6562816-04:00\" xmlns=\"http://docs.oasis-open.org/wsdm/2004/12/mows/wsdm-mows.xsd\"><Request><Message><Size Unit=\"byte\">350</Size><Text>" +
                "&lt;?xml version=\"1.0\" encoding=\"utf-8\"?&gt;&lt;soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"&gt;&lt;soap:Body&gt;&lt;GetCurrentTemperature xmlns=\"http://everest.org/\"&gt;&lt;altitude&gt;8000&lt;/altitude&gt;&lt;/GetCurrentTemperature&gt;&lt;/soap:Body&gt;&lt;/soap:Envelope&gt;\n" +
                "</Text></Message></Request><StateInformation><StateTransition Time=\"2005-04-13T13:34:09.6562816-04:00\" xmlns=\"http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part2.xsd\"><EnteredState><RequestReceivedState xmlns=\"http://docs.oasis-open.org/wsdm/2004/12/mows/wsdm-mows.xsd\" /></EnteredState></StateTransition></StateInformation></RequestProcessingNotification></ManagementEvent></Message></NotificationMessage></Notify></soap:Body></soap:Envelope>" );
        System.out.println( xBean );  // visually verify that both Message elements are serialized with the correct namespaces
    }

    public void testParseURLWithNoTerminatingNewline() throws Exception
    {
        XmlObject.Factory.parse(
                new URL(
                        "file:///C:/Projects/Apache/muse/trunk/src/site/content/interop/src/test/org/apache/xmlbeans/NoNewLine.xml" ) );
        // TODO: try w/ http URL
    }

    public void testXmlObjectValueEquals() throws Exception
    {
        // TODO: test if XmlObject#valueEquals() works when comparing two complex types
    }

    public void testGetDomNode() throws Exception
    {
        SubscribeDocument subscribeDocument = SubscribeDocument.Factory.newInstance();
        SubscribeDocument.Subscribe subscribe = subscribeDocument.addNewSubscribe();
        EndpointReferenceType endpointReferenceType = subscribe.addNewConsumerReference();
        AttributedURI attributedURI = endpointReferenceType.addNewAddress();
        attributedURI.setStringValue( "http://foo.com/" );
        ReferencePropertiesType referencePropertiesType = endpointReferenceType.addNewReferenceProperties();
        XmlBeanUtils.addChildElement( referencePropertiesType, new QName( "Fudge" ) );
        Node domNode = subscribeDocument.getDomNode();
        // TODO: this isn't working - figure out how to reproduce the bug...
        System.out.println( JaxpUtils.toString( domNode ) );
    }

}
