/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws;

import javax.xml.namespace.QName;

/**
 * A category, as defined by part 2 of the MUWS specification.
 */
public interface Category
{
   /**
    * Returns the next most general category in the categorization taxonomy.
    *
    * @return the next most general category in the categorization taxonomy
    *
    * @exception java.util.NoSuchElementException if the category does not have any generalizations
    */
   Category getGeneralization(  );

   /**
    * Returns the name of this category.
    *
    * @return the name of this category
    */
   QName getName(  );

   /**
    * Returns <tt>true</tt> if the category has a generalization. (In other
    * words, returns <tt>true</tt> if <tt>getGeneralization</tt> would return a more
    * general category rather than throwing an exception.)
    *
    * @return <tt>true</tt> if the category
    */
   boolean hasGeneralization(  );
}