/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.impl;

import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.muws.Category;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.CategoryType;
import javax.xml.namespace.QName;

/**
 * An XMLBeans-based implementation of a MUWS {@link Category}.
 */
public class CategoryImpl
   implements Category,
              XmlObjectWrapper
{
   private QName    m_name;
   private Category m_generalization;

   /**
    * Creates a new {@link CategoryImpl} object.
    *
    * @param name DOCUMENT_ME
    */
   public CategoryImpl( QName name )
   {
      m_name = name;
   }

   /**
    * Creates a new {@link CategoryImpl} object.
    *
    * @param name DOCUMENT_ME
    * @param generalization DOCUMENT_ME
    */
   public CategoryImpl( QName    name,
                        Category generalization )
   {
      m_name              = name;
      m_generalization    = generalization;
   }

   /**
    * DOCUMENT_ME
    *
    * @param category DOCUMENT_ME
    */
   public void setGeneralization( Category category )
   {
      m_generalization = category;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Category getGeneralization(  )
   {
      return m_generalization;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getName(  )
   {
      return m_name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public XmlObject getXmlObject(  )
   {
      XmlObject categoryXBean = XmlObject.Factory.newInstance(  );
      XmlObject xBean = categoryXBean;
      for ( Category category = this; category != null; category = category.getGeneralization(  ) )
      {
         xBean = XmlBeanUtils.addChildElement( xBean,
                                               category.getName(  ) );
      }

      return categoryXBean;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean hasGeneralization(  )
   {
      return m_generalization != null;
   }
}