/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.capability;

import org.apache.ws.muws.v1_0.MuwsConstants;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.QueryRelationshipsByTypeDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.QueryRelationshipsByTypeResponseDocument;
import javax.xml.namespace.QName;

/**
 * The MUWS Relationships manageability capability.
 */
public interface RelationshipsCapability
{
   /**
    * Name of this capability.
    */
   String NAME = "Relationships";

   /**
    * Manageability capability URI.
    */
   String URI = MuwsConstants.BASE_URI_MUWS_CAPABILITIES + NAME;

   /**
    * Port type name.
    */
   QName PORT_TYPE_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_WSDL, NAME, MuwsConstants.NSPREFIX_MUWS_PART2_WSDL );

   /**
    * Topic name.
    */
   String TOPIC_NAME = NAME + "Capability";

   /**
    * Relationship property name.
    */
   QName PROP_NAME_RELATIONSHIP =
      new QName( MuwsConstants.NSURI_MUWS_PART2_SCHEMA, "Relationship", MuwsConstants.NSPREFIX_MUWS_PART2_SCHEMA );

   /**
    *  RelationshipCreated subtopic name.
    */
   String SUBTOPIC_NAME_RELATIONSHIP_CREATED = "RelationshipCreated";

   /**
    *  RelationshipDeleted subtopic name.
    */
   String SUBTOPIC_NAME_RELATIONSHIP_DELETED = "RelationshipDeleted";

   /**
    * This operation is a shortcut to query relationships of the same type. See section 4.1.4.1 of the MUWS Part 2 spec.
    *
    * @param requestDoc the request document
    *
    * @return the response document
    */
   QueryRelationshipsByTypeResponseDocument queryRelationshipsByType( QueryRelationshipsByTypeDocument requestDoc );
}