/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.events.impl;

import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.muws.v1_0.events.LangString;
import org.apache.xmlbeans.XmlObject;
import java.util.Locale;

/**
 * A {@link LangString} implementation that supports converting to and from a LangString
 * XMLBean.
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public class LangStringImpl
   implements LangString,
              XmlObjectWrapper
{
   private String m_value;
   private String m_lang;

   /**
    * Creates a new {@link LangStringImpl} object.
    *
    * @param value DOCUMENT_ME
    */
   public LangStringImpl( String value )
   {
      this( value,
            toXmlLang( Locale.getDefault(  ) ) );
   }

   /**
    * Creates a new {@link LangStringImpl} object.
    *
    * @param langString DOCUMENT_ME
    */
   public LangStringImpl( org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString langString )
   {
      this( langString.getStringValue(  ),
            langString.getLang(  ) );
   }

   /**
    * Creates a new {@link LangStringImpl} object.
    *
    * @param value DOCUMENT_ME
    * @param lang DOCUMENT_ME
    */
   public LangStringImpl( String value,
                          String lang )
   {
      m_value    = value;
      m_lang     = lang;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getLang(  )
   {
      return m_lang;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getValue(  )
   {
      return m_value;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public XmlObject getXmlObject(  )
   {
      org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString langString =
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString.Factory.newInstance(  );
      langString.setStringValue( m_value );
      langString.setLang( m_lang );
      return langString;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String toString(  )
   {
      return m_value;
   }

   /**
    * DOCUMENT_ME
    *
    * @param locale DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String toXmlLang( Locale locale )
   {
      if ( "".equals( locale.getLanguage(  ) ) )
      {
         throw new IllegalArgumentException( "Cannot convert a Locale with no language to an xml:lang - RFC 3066 mandates that language tags have a primary subcode." );
      }

      StringBuffer buf = new StringBuffer( locale.getLanguage(  ) );
      if ( !"".equals( locale.getCountry(  ) ) )
      {
         buf.append( "-" );
         buf.append( locale.getCountry(  ) );
         if ( !"".equals( locale.getVariant(  ) ) )
         {
            buf.append( "-" );
            buf.append( locale.getVariant(  ) );
         }
      }

      return buf.toString(  );
   }
}