/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.events.impl;

import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.muws.Category;
import org.apache.ws.muws.v1_0.events.LangString;
import org.apache.ws.muws.v1_0.events.Situation;
import org.apache.ws.muws.v1_0.events.SubstitutableMsg;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationType;
import java.util.Calendar;

/**
 * @author Sal Campana
 */
public class SituationImpl
   implements Situation,
              XmlObjectWrapper
{
   private short            m_priority           = -1;
   private short            m_severity           = -1;
   private LangString       m_message;
   private Calendar         m_situationTime;
   private Boolean          m_successDisposition;
   private SubstitutableMsg m_substitutableMsg;
   private Category         m_situationCategory;

   /**
    * Creates a new {@link SituationImpl} object.
    *
    * @param situationCategory DOCUMENT_ME
    */
   public SituationImpl( Category situationCategory )
   {
      if ( situationCategory == null )
      {
         throw new IllegalArgumentException( "Category may not be null." );
      }

      m_situationCategory = situationCategory;
   }

   /**
    * DOCUMENT_ME
    *
    * @param message DOCUMENT_ME
    */
   public void setMessage( LangString message )
   {
      m_message = message;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public LangString getMessage(  )
   {
      return m_message;
   }

   /**
    * DOCUMENT_ME
    *
    * @param priority DOCUMENT_ME
    */
   public void setPriority( short priority )
   {
      if ( ( priority < 0 ) || ( priority > 100 ) )
      {
         throw new IllegalArgumentException( "Priority may only be a value from 0 to 100." );
      }

      m_priority = priority;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public short getPriority(  )
   {
      return m_priority;
   }

   /**
    * DOCUMENT_ME
    *
    * @param severity DOCUMENT_ME
    */
   public void setSeverity( short severity )
   {
      if ( ( severity < 0 ) || ( severity > 6 ) )
      {
         throw new IllegalArgumentException( "Severity may only be a value from 0 to 6." );
      }

      m_severity = severity;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public short getSeverity(  )
   {
      return m_severity;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Category getSituationCategory(  )
   {
      return m_situationCategory;
   }

   /**
    * DOCUMENT_ME
    *
    * @param cal DOCUMENT_ME
    */
   public void setSituationTime( Calendar cal )
   {
      m_situationTime = cal;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Calendar getSituationTime(  )
   {
      return m_situationTime;
   }

   /**
    * DOCUMENT_ME
    *
    * @param msg DOCUMENT_ME
    */
   public void setSubstitutionalMsg( SubstitutableMsg msg )
   {
      m_substitutableMsg = msg;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public SubstitutableMsg getSubstitutionalMsg(  )
   {
      return m_substitutableMsg;
   }

   /**
    * DOCUMENT_ME
    *
    * @param bool DOCUMENT_ME
    */
   public void setSuccessDisposition( Boolean bool )
   {
      m_successDisposition = bool;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Boolean getSuccessDisposition(  )
   {
      return m_successDisposition;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public XmlObject getXmlObject(  )
   {
      SituationDocument                                                          situationDoc =
         SituationDocument.Factory.newInstance(  );

      org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationType         situationType         =
         situationDoc.addNewSituation(  );
      org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SituationCategoryType situationCategoryType =
         situationType.addNewSituationCategory(  );
      XmlBeanUtils.addChildElement( situationCategoryType,
                                    ( (XmlObjectWrapper) m_situationCategory ).getXmlObject(  ) );

      if ( m_message != null )
      {
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString langString =
            (org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.LangString) ( (XmlObjectWrapper) m_message )
                        .getXmlObject(  );
         situationType.setMessage( langString );
      }

      if ( m_substitutableMsg != null )
      {
         org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SubstitutableMsgType substitutableMsg =
            (org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.SubstitutableMsgType) ( (XmlObjectWrapper) m_substitutableMsg )
                        .getXmlObject(  );
         situationType.setSubstitutableMsg( substitutableMsg );
      }

      if ( m_priority != -1 )
      {
         situationType.setPriority( m_priority );
      }

      if ( m_severity != -1 )
      {
         situationType.setSeverity( m_severity );
      }

      if ( m_situationTime != null )
      {
         situationType.setSituationTime( m_situationTime );
      }

      if ( m_successDisposition != null )
      {
         if ( m_successDisposition.booleanValue(  ) )
         {
            situationType.setSuccessDisposition( SituationType.SuccessDisposition.SUCCESSFUL );
         }
         else
         {
            situationType.setSuccessDisposition( SituationType.SuccessDisposition.UNSUCCESSFUL );
         }
      }

      return situationDoc;
   }
}