/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.events.impl;

import org.apache.commons.id.IdentifierUtils;
import org.apache.commons.id.uuid.UUID;
import org.apache.ws.XmlObjectWrapper;
import org.apache.ws.muws.v1_0.events.ManagementEvent;
import org.apache.ws.muws.v1_0.events.Situation;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ComponentAddressType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManagementEventDocument;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * @author Sal Campana
 */
public class XmlBeansManagementEvent
   implements XmlObjectWrapper,
              ManagementEvent
{
   private String    m_reporterResourceId;
   private String    m_sourceResourceId;
   private String    m_eventId;
   private Object[]  m_reporterComponentAddresses;
   private Object[]  m_sourceComponentAddresses;
   private Object[]  m_any;
   private Situation m_situation;

   /**
    * Creates a new {@link XmlBeansManagementEvent} object.
    *
    * @param situation DOCUMENT_ME
    */
   public XmlBeansManagementEvent( Situation situation )
   {
      if ( situation == null )
      {
         throw new IllegalArgumentException( "Situation may not be null." );
      }

      m_situation = situation;
   }

   /**
    * DOCUMENT_ME
    *
    * @param any DOCUMENT_ME
    */
   public void setAny( Object[] any )
   {
      m_any = any;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object[] getAny(  )
   {
      return m_any;
   }

   /**
    * DOCUMENT_ME
    *
    * @param eventId DOCUMENT_ME
    */
   public void setEventId( String eventId )
   {
      m_eventId = eventId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getEventId(  )
   {
      return m_eventId;
   }

   /**
    * DOCUMENT_ME
    *
    * @param reporterComponentAddresses DOCUMENT_ME
    */
   public void setReporterComponentAddresses( Object[] reporterComponentAddresses )
   {
      m_reporterComponentAddresses = reporterComponentAddresses;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object[] getReporterComponentAddresses(  )
   {
      return m_reporterComponentAddresses;
   }

   /**
    * DOCUMENT_ME
    *
    * @param reporterResourceId DOCUMENT_ME
    */
   public void setReporterResourceId( String reporterResourceId )
   {
      m_reporterResourceId = reporterResourceId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getReporterResourceId(  )
   {
      return m_reporterResourceId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Situation getSituation(  )
   {
      return m_situation;
   }

   /**
    * DOCUMENT_ME
    *
    * @param sourceComponentAddresses DOCUMENT_ME
    */
   public void setSourceComponentAddresses( Object[] sourceComponentAddresses )
   {
      m_sourceComponentAddresses = sourceComponentAddresses;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Object[] getSourceComponentAddresses(  )
   {
      return m_sourceComponentAddresses;
   }

   /**
    * DOCUMENT_ME
    *
    * @param sourceResourceId DOCUMENT_ME
    */
   public void setSourceResourceId( String sourceResourceId )
   {
      m_sourceResourceId = sourceResourceId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getSourceResourceId(  )
   {
      return m_sourceResourceId;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public XmlObject getXmlObject(  )
   {
      return buildManagementEventNotifDocXmlBean(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected XmlObject buildManagementEventNotifDocXmlBean(  )
   {
      ManagementEventDocument                                                  managementEventDocument =
         ManagementEventDocument.Factory.newInstance(  );
      org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManagementEventType managementEventType =
         managementEventDocument.addNewManagementEvent(  );

      org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ComponentType       reporterComponentType =
         managementEventType.addNewReporterComponent(  );
      if ( m_reporterResourceId != null )
      {
         reporterComponentType.setResourceId( m_reporterResourceId );
      }

      if ( m_reporterComponentAddresses != null )
      {
         List reporterComponentAddresses = new ArrayList(  );

         for ( int i = 0; i < m_reporterComponentAddresses.length; i++ )
         {
            ComponentAddressType reporterComponentAddressType = reporterComponentType.addNewComponentAddress(  );
            XmlBeanUtils.addChildElement( reporterComponentAddressType, (XmlObject) m_reporterComponentAddresses[i] );
            reporterComponentAddresses.add( reporterComponentAddressType );
         }

         reporterComponentType.setComponentAddressArray( (ComponentAddressType[]) reporterComponentAddresses
                                                           .toArray( new ComponentAddressType[0] ) );
      }

      org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ComponentType sourceComponentType =
         managementEventType.addNewSourceComponent(  );
      if ( m_sourceResourceId != null )
      {
         sourceComponentType.setResourceId( m_sourceResourceId );
      }

      if ( m_sourceComponentAddresses != null )
      {
         List sourceComponentAddresses = new ArrayList(  );
         for ( int i = 0; i < m_sourceComponentAddresses.length; i++ )
         {
            ComponentAddressType sourceComponentAddressType = sourceComponentType.addNewComponentAddress(  );
            XmlBeanUtils.addChildElement( sourceComponentAddressType, (XmlObject) m_sourceComponentAddresses[i] );
            sourceComponentAddresses.add( sourceComponentAddressType );
         }

         sourceComponentType.setComponentAddressArray( (ComponentAddressType[]) sourceComponentAddresses.toArray( new ComponentAddressType[0] ) );
      }

      if ( m_eventId != null )
      {
         managementEventType.setEventId( m_eventId );
      }
      else
      {
         managementEventType.setEventId( ( (UUID) IdentifierUtils.UUID_VERSION_FOUR_GENERATOR.nextIdentifier(  ) )
                                         .toString(  ) );
      }

      managementEventType.setReportTime( Calendar.getInstance(  ) );

      if ( m_any != null )
      {
         for ( int i = 0; i < m_any.length; i++ )
         {
            XmlBeanUtils.addChildElement( managementEventType, (XmlObject) m_any[i] );
         }
      }

      XmlBeanUtils.addChildElement( managementEventType,
                                    ( (XmlObjectWrapper) m_situation ).getXmlObject(  ) );
      return managementEventDocument;
   }
}