/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.impl.advertiser;

import org.apache.ws.resource.impl.AbstractResourceHome;

import javax.xml.namespace.QName;
import java.io.Serializable;
import java.util.Map;

/**
 * Home for ResourceAdvertiser WS-Resources.
 */
public class ResourceAdvertiserHome
        extends AbstractResourceHome
        implements Serializable
{

    /**
     * The service endpoint name as registered with the SOAP Platform.  This is useful for building EPR's. *
     */
    public static final QName SERVICE_NAME =
            javax.xml.namespace.QName.valueOf(
                    "{http://ws.apache.org/muws/advertiser/base/service/}ResourceAdvertiser" );

    /**
     * The management PortType associated with this resource. This is useful for building EPR's.*
     */
    public static final QName PORT_TYPE =
            javax.xml.namespace.QName.valueOf(
                    "{http://ws.apache.org/muws/advertiser/base/service/}AdvertiserPortType" );

    /**
     * The WSDL Port name associated with the resource. This is useful for building EPR's. *
     */
    public static final String PORT_NAME = "ResourceAdvertiser";

    /**
     * A NamespaceVerionHolder which maintains the QNames of Spec Wsdls
     */
    public static final org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl SPEC_NAMESPACE_SET =
            new org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl();

    /**
     * A constant for the JNDI Lookup name for this home. *
     */
    public static final String HOME_LOCATION =
            org.apache.ws.resource.JndiConstants.CONTEXT_NAME_SERVICES + "/" + SERVICE_NAME.getLocalPart() + "/"
            + org.apache.ws.resource.JndiConstants.ATOMIC_NAME_HOME;

    /**
     * Map containing all ResourceAdvertiserResource instances - this map <em>must</em> be static for compatibility with
     * certain JNDI providers.
     */
    private static Map s_resources;

    public synchronized void init() throws Exception
    {
        if ( !isInitialized() )
        {
            super.init();
            ResourceAdvertiserResource resourceAdvertiser = new ResourceAdvertiserResource();
            resourceAdvertiser.init();
            add( resourceAdvertiser );
        }
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public org.apache.ws.resource.properties.NamespaceVersionHolder getNamespaceVersionHolder()
    {
        return SPEC_NAMESPACE_SET;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public QName getPortType()
    {
        return PORT_TYPE;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public QName getServiceName()
    {
        return SERVICE_NAME;
    }

    /**
     * DOCUMENT_ME
     *
     * @return DOCUMENT_ME
     */
    public String getServicePortName()
    {
        return PORT_NAME;
    }

    /**
     * Returns a map of all FilesystemResource instances. Used by the {@link AbstractResourceHome} superclass.
     */
    protected final synchronized Map getResourceMap()
    {
        if ( s_resources == null )
        {
            s_resources = AbstractResourceHome.createResourceMap( m_resourceIsPersistent );
        }
        return s_resources;
    }

}
