/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.impl.advertiser;

import org.apache.ws.muws.v1_0.MuwsConstants;
import org.apache.ws.muws.v1_0.capability.AdvertisementCapability;
import org.apache.ws.muws.v1_0.topics.impl.XmlBeansAdvertisementTopicImpl;
import org.apache.ws.muws.MuwsUtils;
import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicSpace;
import org.apache.ws.notification.topics.impl.TopicSpaceImpl;
import org.apache.ws.resource.ResourceCreationListener;
import org.apache.ws.resource.ResourceDestructionListener;
import org.apache.ws.resource.impl.AbstractResourceHome;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import java.util.ArrayList;
import java.util.List;

/**
 * A ResourceAdvertiser WS-Resource.
 * <p/>
 * NOTE: This class is generated but IS meant to be modified.
 */
public class ResourceAdvertiserResource
        extends AbstractResourceAdvertiserResource
{
    /**
     * A NamespaceVerionHolder which maintains the QNames of Spec Wsdls
     */
    public static final org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl SPEC_NAMESPACE_SET =
            new org.apache.ws.muws.v1_0.impl.WsdmNamespaceVersionHolderImpl();

    /**
     * Creates a new {@link ResourceAdvertiserResource} object.
     */
    public ResourceAdvertiserResource()
    {
        setID( null );  // singleton
    }

    /**
     * Initializes this resource's state (properties, etc.).
     */
    public void init()
    {
        super.init();

        /**
         * The ResourcePropertySet which contains all the defined ResourceProperties
         */
        org.apache.ws.resource.properties.ResourcePropertySet resourcePropertySet = getResourcePropertySet();
        org.apache.ws.resource.properties.ResourceProperty resourceProperty = null;

        try
        {
            // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd}ManageabilityCapability Resource Property
            resourceProperty = resourcePropertySet.get( ResourceAdvertiserPropertyQNames.MANAGEABILITYCAPABILITY );
            org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument prop_manageabilitycapability =
                    org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManageabilityCapabilityDocument.Factory
                    .newInstance();
            prop_manageabilitycapability.setManageabilityCapability( AdvertisementCapability.URI );
            resourceProperty.add( prop_manageabilitycapability );

            // init the {http://docs.oasis-open.org/wsdm/2004/12/muws/wsdm-muws-part1.xsd}ResourceId Resource Property
            MuwsUtils.initResourceIdProperty( this );

            org.apache.ws.notification.topics.TopicSpaceSet topicSpaceSet = getTopicSpaceSet();
            TopicSpace topicSpace =
                    new TopicSpaceImpl( MuwsConstants.NSURI_MUWS_PART2_TOPICS );

            Topic manageabilityEndpointCreationTopic =
                    new XmlBeansAdvertisementTopicImpl(
                            AdvertisementCapability.TOPIC_MANAGEABILITY_ENPOINT_CREATION_NAME,
                            false );
            Topic manageabilityEndpointDestructionTopic =
                    new XmlBeansAdvertisementTopicImpl(
                            AdvertisementCapability.TOPIC_MANAGEABILITY_ENPOINT_DESTRUCTION_NAME,
                            false );

            XmlBeansAdvertisementTopicImpl manageableResourceCreationTopic =
                    new XmlBeansAdvertisementTopicImpl(
                            AdvertisementCapability.TOPIC_MANAGEABLE_RESOURCE_CREATION_NAME );
            XmlBeansAdvertisementTopicImpl manageableResourceDestructionTopic =
                    new XmlBeansAdvertisementTopicImpl(
                            AdvertisementCapability.TOPIC_MANAGEABLE_RESOURCE_DESTRUCTION_NAME );

            manageabilityEndpointCreationTopic.addTopic( manageableResourceCreationTopic );
            manageabilityEndpointDestructionTopic.addTopic( manageableResourceDestructionTopic );
            topicSpace.addTopic( manageabilityEndpointCreationTopic );
            topicSpace.addTopic( manageabilityEndpointDestructionTopic );
            topicSpaceSet.addTopicSpace( topicSpace );

            List homeNames = findHomes( new InitialContext(),
                    new ArrayList(),
                    null );
            addResourceLifecycleListenersToHomes( homeNames, manageableResourceCreationTopic,
                    manageableResourceDestructionTopic );

        }
        catch ( Exception e )
        {
            throw new RuntimeException( "There was a problem in initializing your resource properties.  Please check your init() method. Cause: "
                    + e.getLocalizedMessage() );
        }
    }

    /**
     * Locates the JNDI-deployed homes and returns a list containing their lookup strings. Calls itself recursively.
     *
     * @param context
     * @param homes
     * @param fullName
     *
     * @return A list of homes.
     *
     * @throws NamingException
     */
    private List findHomes( Context context,
                            List homes,
                            String fullName )
            throws NamingException
    {
        NamingEnumeration bindingEnum = context.listBindings( "" );
        while ( bindingEnum.hasMore() )
        {
            Binding binding = (Binding) bindingEnum.next();
            String name = binding.getName();
            Object value = binding.getObject();
            if ( name.equals( "home" ) )
            {
                homes.add( fullName == null ? name : fullName + "/" + name );
            }
            else if ( value instanceof Context )
            {
                findHomes( (Context) value, homes, fullName == null ? name : fullName + "/" + name );
            }
        }
        return homes;
    }

    private void addResourceLifecycleListenersToHomes( List homeNames,
                                                       ResourceCreationListener resourceCreationListener,
                                                       ResourceDestructionListener resourceDestructionListener )
            throws NamingException
    {
        for ( int i = 0; i < homeNames.size(); i++ )
        {
            String homeName = (String) homeNames.get( i );
            if ( !homeName.equals( ResourceAdvertiserHome.HOME_LOCATION ) )  // do not register its own home
            {
                AbstractResourceHome home = (AbstractResourceHome) new InitialContext().lookup( homeName );
                home.addResourceCreationListener( resourceCreationListener );
                home.addResourceDestructionListener( resourceDestructionListener );
            }
        }
    }

}
