/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.v1_0.topics;

import org.apache.ws.mows.v1_0.capability.OperationalStateCapability;
import org.apache.ws.muws.v1_0.MuwsConstants;
import org.apache.ws.muws.v1_0.capability.AdvertisementCapability;
import org.apache.ws.muws.v1_0.capability.ConfigurationCapability;
import org.apache.ws.muws.v1_0.capability.CorrelatablePropertiesCapability;
import org.apache.ws.muws.v1_0.capability.DescriptionCapability;
import org.apache.ws.muws.v1_0.capability.IdentityCapability;
import org.apache.ws.muws.v1_0.capability.ManageabilityCharacteristicsCapability;
import org.apache.ws.muws.v1_0.capability.MetricsCapability;
import org.apache.ws.muws.v1_0.capability.RelationshipAccessCapability;
import org.apache.ws.muws.v1_0.capability.RelationshipResourceCapability;
import org.apache.ws.muws.v1_0.capability.RelationshipsCapability;
import org.apache.ws.muws.v1_0.capability.StateCapability;
import javax.xml.namespace.QName;

/**
 * A Constant class for maintaining one place to look for the Topic QNames
 * defined in the Muws Spec.
 *
 * This is useful for registering for notifs from these topics.
 *
 * This class utilizes the fields from the Capability interfaces and is meant
 * to make a collection of the Topic qnames.
 *
 * @author Sal Campana
 */
public interface MuwsTopicsConstants
{
   /** DOCUMENT_ME */
   QName MUWS_TOPICSPACE_NAME = new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, "MuwsNotificationTopics" );

   /** DOCUMENT_ME */
   QName IDENTITYCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, IdentityCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName MANAGEABILITYCHARACTERISTICSCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, ManageabilityCharacteristicsCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName CORRELATABLEPROPERTIESCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, CorrelatablePropertiesCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName DESCRIPTIONCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, DescriptionCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName STATE_TOPIC_NAME = new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, StateCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName OPERATIONALSTATECAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, OperationalStateCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName METRICSCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, MetricsCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName CONFIGURATIONCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, ConfigurationCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName RELATIONSHIPSCAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, RelationshipsCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName RELATIONSHIP_CREATED_SUBTOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, RelationshipsCapability.SUBTOPIC_NAME_RELATIONSHIP_CREATED );

   /** DOCUMENT_ME */
   QName RELATIONSHIP_DELETED_SUBTOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, RelationshipsCapability.SUBTOPIC_NAME_RELATIONSHIP_DELETED );

   /** DOCUMENT_ME */
   QName RELATIONSHIP_ACCESS_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, RelationshipAccessCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName RELATIONSHIPRESOURCECAPABILITY_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, RelationshipResourceCapability.TOPIC_NAME );

   /** DOCUMENT_ME */
   QName MANAGEABILITY_ENPOINT_CREATION_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS,
                 AdvertisementCapability.TOPIC_MANAGEABILITY_ENPOINT_CREATION_NAME );

   /** DOCUMENT_ME */
   QName MANAGEABLE_RESOURCE_CREATION_SUBTOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS,
                 AdvertisementCapability.TOPIC_MANAGEABLE_RESOURCE_CREATION_NAME );

   /** DOCUMENT_ME */
   QName MANAGEABILITY_ENPOINT_DESTRUCTION_TOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS,
                 AdvertisementCapability.TOPIC_MANAGEABILITY_ENPOINT_DESTRUCTION_NAME );

   /** DOCUMENT_ME */
   QName MANAGEABLE_RESOURCE_DESTRUCTION_SUBTOPIC_NAME =
      new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS,
                 AdvertisementCapability.TOPIC_MANAGEABLE_RESOURCE_DESTRUCTION_NAME );
}